<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Maintenance.mtc.test.php 4716 2006-04-20 16:56:02Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Table/Core.php';
require_once MAX_PATH . '/lib/max/Maintenance.php';
require_once 'Date.php';

/**
 * A class for testing the Maintenance class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@m3.net>
 */
class Maintenance_TestOfMaintenance extends UnitTestCase
{

    /**
     * The constructor method.
     */
    function Maintenance_TestOfMaintenance()
    {
        $this->UnitTestCase();
        Mock::generate('MAX_Dal_Maintenance');
        $oDal = &new MockMAX_Dal_Maintenance($this);
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->register('MAX_Dal_Maintenance', $oDal);
    }

    /**
     * Tests that date ranges can be correctly converted into operation
     * interval IDs.
     */
    function testConvertDateRangeToOperationIntervalID()
    {
        $maintenance = &new Maintenance();
        // Test the first operation interval ID range in the week the test was written,
        // using a default operation interval of 60 minutes
        $start = new Date('2004-08-15 00:00:00');
        $end = new Date('2004-08-15 00:59:59');
        $result = $maintenance->convertDateRangeToOperationIntervalID($start, $end, 60);
        $this->assertEqual($result, 0);
        // Test the same range with a new operation interval of 30 minutes to make
        // sure that the range is recognised as spanning two operation interval IDs
        $result = $maintenance->convertDateRangeToOperationIntervalID($start, $end, 30);
        $this->assertFalse($result);
        // Test the second operation interval ID range in the week the test was written,
        // using an operation interval of 30 minutes, and a non-definative date range
        $start = new Date('2004-08-15 00:35:00');
        $end = new Date('2004-08-15 00:40:00');
        $result = $maintenance->convertDateRangeToOperationIntervalID($start, $end, 30);
        $this->assertEqual($result, 1);
    }

    /**
     * Tests that dates can be correctly converted into opertation
     * interval IDs.
     */
    function testConvertDateToOperationIntervalID()
    {
        $maintenance = &new Maintenance();
        // Test a date in the first operation interval ID in the week before the test was
        // written, using a default operation interval of 60 minutes
        $date = new Date('2004-08-08 00:40:00');
        $result = $maintenance->convertDateToOperationIntervalID($date, 60);
        $this->assertEqual($result, 0);
        // Test a date in the last operation interval ID in the week before the test was
        // written, using an operation interval of 30 minutes
        $date = new Date('2004-08-14 23:40:00');
        $result = $maintenance->convertDateToOperationIntervalID($date, 30);
        $this->assertEqual($result, 335);
    }

    /**
     * Tests that dates can be correctly converted into the start and end
     * dates of the corresponding operation interval ID.
     */
    function testConvertDateToOperationIntervalStartAndEndDates()
    {
        $maintenance = &new Maintenance();
        // Test a date in the first operation interval ID in the week before the test was
        // written, using a default operation interval of 60 minutes
        $date = new Date('2004-08-08 00:40:00');
        list($start, $end) = $maintenance->convertDateToOperationIntervalStartAndEndDates($date, 60);
        $this->assertEqual($start, new Date('2004-08-08 00:00:00'));
        $this->assertEqual($end, new Date('2004-08-08 00:59:59'));
        // Test the same date, but with an operation interval of 30 minutes
        list($start, $end) = $maintenance->convertDateToOperationIntervalStartAndEndDates($date, 30);
        $this->assertEqual($start, new Date('2004-08-08 00:30:00'));
        $this->assertEqual($end, new Date('2004-08-08 00:59:59'));
    }

    /**
     * Tests the calculation of the previous operation interval.
     */
    function testPreviousOperationIntervalID()
    {
        $maintenance = &new Maintenance();
        $operationIntervalID = 1;
        $operationInterval = 60;
        $operationIntervalID = $maintenance->previousOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 0);
        $operationIntervalID = $maintenance->previousOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 167);
        $operationIntervalID = $maintenance->previousOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 166);
        $operationIntervalID = 1;
        $operationInterval = 30;
        $operationIntervalID = $maintenance->previousOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 0);
        $operationIntervalID = $maintenance->previousOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 335);
        $operationIntervalID = $maintenance->previousOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 334);
    }

    /**
     * Tests the calculation of the next operation interval.
     */
    function testNextOperationIntervalID()
    {
        $maintenance = &new Maintenance();
        $operationIntervalID = 166;
        $operationInterval = 60;
        $operationIntervalID = $maintenance->nextOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 167);
        $operationIntervalID = $maintenance->nextOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 0);
        $operationIntervalID = $maintenance->nextOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 1);
        $operationIntervalID = 334;
        $operationInterval = 30;
        $operationIntervalID = $maintenance->nextOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 335);
        $operationIntervalID = $maintenance->nextOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 0);
        $operationIntervalID = $maintenance->nextOperationIntervalID($operationIntervalID, $operationInterval);
        $this->assertEqual($operationIntervalID, 1);
    }

    /**
     * Tests the calulation of a range of operation intervals.
     */
    function testGetOperationIntervalRange()
    {
        $maintenance = &new Maintenance();
        $range = array();
        $date = new Date('2004-09-11 19:00:00');
        $maintenance->getOperationIntervalRange($range, $date, 5, 60);
        $this->assertNull($range[162]);
        $this->assertEqual($range[163]['start'], new Date('2004-09-11 19:00:00'));
        $this->assertEqual($range[163]['end'], new Date('2004-09-11 19:59:59'));
        $this->assertEqual($range[164]['start'], new Date('2004-09-11 20:00:00'));
        $this->assertEqual($range[164]['end'], new Date('2004-09-11 20:59:59'));
        $this->assertEqual($range[165]['start'], new Date('2004-09-11 21:00:00'));
        $this->assertEqual($range[165]['end'], new Date('2004-09-11 21:59:59'));
        $this->assertEqual($range[166]['start'], new Date('2004-09-11 22:00:00'));
        $this->assertEqual($range[166]['end'], new Date('2004-09-11 22:59:59'));
        $this->assertEqual($range[167]['start'], new Date('2004-09-11 23:00:00'));
        $this->assertEqual($range[167]['end'], new Date('2004-09-11 23:59:59'));
        $this->assertNull($range[0]);
    }

    /**
     * Tests the test of dates in the same hour.
     */
    function testCheckDatesInSameHour()
    {
        $maintenance = &new Maintenance();
        $start = new Date('2004-09-11 19:00:00');
        $end   = new Date('2004-09-11 19:00:00');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertTrue($return);
        $start = new Date('2004-09-11 19:59:59');
        $end   = new Date('2004-09-11 19:59:59');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertTrue($return);
        $start = new Date('2004-09-11 19:00:00');
        $end   = new Date('2004-09-11 19:00:01');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertTrue($return);
        $start = new Date('2004-09-11 19:00:00');
        $end   = new Date('2004-09-11 19:59:59');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertTrue($return);
        $start = new Date('2004-09-11 19:59:59');
        $end   = new Date('2004-09-11 20:00:00');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertFalse($return);
        $start = new Date('2004-09-11 18:00:00');
        $end   = new Date('2004-09-12 18:00:00');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertFalse($return);
        $start = new Date('2004-08-11 18:00:00');
        $end   = new Date('2004-09-11 18:00:00');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertFalse($return);
        $start = new Date('2003-09-11 18:00:00');
        $end   = new Date('2004-09-11 18:00:00');
        $return = $maintenance->checkDatesInSameHour($start, $end);
        $this->assertFalse($return);
    }

    /**
     * Tests the checkIntervalDates() method.
     */
    function testCheckIntervalDates()
    {
        $maintenance = &new Maintenance();
        $conf = &$GLOBALS['_MAX']['CONF'];
        // Set the operation interval
        $conf['maintenance']['operationInterval'] = 30;
        // Test less than one operation interval
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:15:00');
        $this->assertFalse($maintenance->checkIntervalDates($start, $end));
        // Test more than one operation inteterval
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:45:00');
        $this->assertFalse($maintenance->checkIntervalDates($start, $end));
        // Test exactly one operation interval
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:29:59');
        $this->assertTrue($maintenance->checkIntervalDates($start, $end));
        // Set the operation interval
        $conf['maintenance']['operationInterval'] = 60;
        // Test less than one operation interval/hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:30:00');
        $this->assertFalse($maintenance->checkIntervalDates($start, $end));
        // Test more than one operation inteterval/hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 01:15:00');
        $this->assertFalse($maintenance->checkIntervalDates($start, $end));
        // Test exactly one operation interval/hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:59:59');
        $this->assertTrue($maintenance->checkIntervalDates($start, $end));
        // Set the operation interval
        $conf['maintenance']['operationInterval'] = 120;
        // Test less than one hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:15:00');
        $this->assertFalse($maintenance->checkIntervalDates($start, $end));
        // Test more than one hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 04:00:00');
        $this->assertFalse($maintenance->checkIntervalDates($start, $end));
        // Test exactly one hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:59:59');
        $this->assertTrue($maintenance->checkIntervalDates($start, $end));
    }

    /**
     * Tests the maxConnectionWindows() method.
     *
     * @TODO Might be worth re-coding the mocked actor code if
     *       simpletest can support returning at with parameter
     *       values...
     */
    function testMaxConnectionWindows()
    {
        $maintenance = new Maintenance();
        // Allow mock objects of the MAX_Dal_Maintenance class
        Mock::generate('MAX_Dal_Maintenance');
        // Generate a mock object of the database connection
        $dbh = new MockMAX_Dal_Maintenance($this);
        // Set up the test values
        $dbh->setReturnValueAt(0, 'maxWindow', 0);
        $dbh->setReturnValueAt(1, 'maxWindow', 0);
        $dbh->setReturnValueAt(2, 'maxWindow', 0);
        $dbh->setReturnValueAt(3, 'maxWindow', 60);
        $dbh->setReturnValueAt(4, 'maxWindow', 70);
        $dbh->setReturnValueAt(5, 'maxWindow', 60);
        $dbh->setReturnValueAt(6, 'maxWindow', 90);
        $dbh->setReturnValueAt(7, 'maxWindow', 100);
        // Register the mock object with the service locator
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->register('MAX_Dal_Maintenance', $dbh);
        // Test
        list($impressionWindow, $clickWindow) = $maintenance->maxConnectionWindows();
        $this->assertEqual($impressionWindow, 0);
        $this->assertEqual($clickWindow, 0);
        // Test
        list($impressionWindow, $clickWindow) = $maintenance->maxConnectionWindows();
        $this->assertEqual($impressionWindow, 0);
        $this->assertEqual($clickWindow, 60);
        // Test
        list($impressionWindow, $clickWindow) = $maintenance->maxConnectionWindows();
        $this->assertEqual($impressionWindow, 70);
        $this->assertEqual($clickWindow, 60);
        // Test
        list($impressionWindow, $clickWindow) = $maintenance->maxConnectionWindows();
        $this->assertEqual($impressionWindow, 90);
        $this->assertEqual($clickWindow, 100);
        $dbh->tally();
    }

    /**
     * Tests that an e-mail advising a campaign has been activated is able to
     * be generated correctly.
     */
    function testSendActivateCampaignEmail()
    {
        $maintenance = &new Maintenance();
        $contactName = 'Andrew Hill';
        $campaignName = 'Test Activation Campaign';
        $ads[0] = array('First Test Banner', '', 'http://example.com/');
        $ads[1] = array('Second Test Banner', 'Alt. Description', 'http://example.com/');
        $ads[2] = array('', 'Third Test Banner', 'http://example.com/foo/bar.html');
        $ads[3] = array('', '', 'http://example.com/foo.html');
        $email = $maintenance->prepareActivateCampaignEmail($contactName, $campaignName, $ads);
        $desiredEmail  = 'Dear Andrew Hill,' . "\n\n";
        $desiredEmail .= 'The following ads have been activated because '. "\n";
        $desiredEmail .= 'the campaign activation date has been reached.' . "\n\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 0] First Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 1] Second Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 2] Third Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo/bar.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 3] Untitled' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= "\nThank you for advertising with us.\n\n";
        $desiredEmail .= "Regards,\n\n";
        $desiredEmail .= $conf['email']['admin_name'];
        $this->assertEqual($email, $desiredEmail);
    }

    /**
     * Tests that e-mails advising a campaign has been deactivated is able to
     * be generated correctly.
     */
    function testSendDeactivateCampaignEmail()
    {
        $maintenance = &new Maintenance();
        $contactName = 'Andrew Hill';
        $campaignName = 'Test Activation Campaign';
        $ads[0] = array('First Test Banner', '', 'http://example.com/');
        $ads[1] = array('Second Test Banner', 'Alt. Description', 'http://example.com/');
        $ads[2] = array('', 'Third Test Banner', 'http://example.com/foo/bar.html');
        $ads[3] = array('', '', 'http://example.com/foo.html');
        $email = $maintenance->prepareDeactivateCampaignEmail($contactName, $campaignName, 2, $ads);
        $desiredEmail  = 'Dear Andrew Hill,' . "\n\n";
        $desiredEmail .= 'The following ads have been disabled because:' . "\n";
        $desiredEmail .= '  - There are no impressions remaining' . "\n\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 0] First Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 1] Second Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 2] Third Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo/bar.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 3] Untitled' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= "\nIf you would like to continue advertising on our website,\n";
        $desiredEmail .= "please feel free to contact us.\n";
        $desiredEmail .= "We'd be glad to hear from you.\n\n";
        $desiredEmail .= "Regards,\n\n";
        $desiredEmail .= $conf['email']['admin_name'];
        $this->assertEqual($email, $desiredEmail);
        $email = $maintenance->prepareDeactivateCampaignEmail($contactName, $campaignName, 4, $ads);
        $desiredEmail  = 'Dear Andrew Hill,' . "\n\n";
        $desiredEmail .= 'The following ads have been disabled because:' . "\n";
        $desiredEmail .= '  - There are no clicks remaining' . "\n\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 0] First Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 1] Second Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 2] Third Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo/bar.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 3] Untitled' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= "\nIf you would like to continue advertising on our website,\n";
        $desiredEmail .= "please feel free to contact us.\n";
        $desiredEmail .= "We'd be glad to hear from you.\n\n";
        $desiredEmail .= "Regards,\n\n";
        $desiredEmail .= $conf['email']['admin_name'];
        $this->assertEqual($email, $desiredEmail);
        $email = $maintenance->prepareDeactivateCampaignEmail($contactName, $campaignName, 8, $ads);
        $desiredEmail  = 'Dear Andrew Hill,' . "\n\n";
        $desiredEmail .= 'The following ads have been disabled because:' . "\n";
        $desiredEmail .= '  - There are no conversions remaining' . "\n\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 0] First Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 1] Second Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 2] Third Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo/bar.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 3] Untitled' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= "\nIf you would like to continue advertising on our website,\n";
        $desiredEmail .= "please feel free to contact us.\n";
        $desiredEmail .= "We'd be glad to hear from you.\n\n";
        $desiredEmail .= "Regards,\n\n";
        $desiredEmail .= $conf['email']['admin_name'];
        $this->assertEqual($email, $desiredEmail);
        $email = $maintenance->prepareDeactivateCampaignEmail($contactName, $campaignName, 16, $ads);
        $desiredEmail  = 'Dear Andrew Hill,' . "\n\n";
        $desiredEmail .= 'The following ads have been disabled because:' . "\n";
        $desiredEmail .= '  - The campaign deactivation date has been reached' . "\n\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 0] First Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 1] Second Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 2] Third Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo/bar.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 3] Untitled' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= "\nIf you would like to continue advertising on our website,\n";
        $desiredEmail .=  "please feel free to contact us.\n";
        $desiredEmail .= "We'd be glad to hear from you.\n\n";
        $desiredEmail .= "Regards,\n\n";
        $desiredEmail .= $conf['email']['admin_name'];
        $this->assertEqual($email, $desiredEmail);
        $value = 0 | 2 | 4 | 8 | 16;
        $email = $maintenance->prepareDeactivateCampaignEmail($contactName, $campaignName, $value, $ads);
        $desiredEmail  = 'Dear Andrew Hill,' . "\n\n";
        $desiredEmail .= 'The following ads have been disabled because:' . "\n";
        $desiredEmail .= '  - There are no impressions remaining' . "\n";
        $desiredEmail .= '  - There are no clicks remaining' . "\n";
        $desiredEmail .= '  - There are no conversions remaining' . "\n";
        $desiredEmail .= '  - The campaign deactivation date has been reached' . "\n\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 0] First Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 1] Second Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 2] Third Test Banner' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo/bar.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= 'Ad [ID 3] Untitled' . "\n";
        $desiredEmail .= 'Linked to: http://example.com/foo.html' . "\n";
        $desiredEmail .= "-------------------------------------------------------\n";
        $desiredEmail .= "\nIf you would like to continue advertising on our website,\n";
        $desiredEmail .=  "please feel free to contact us.\n";
        $desiredEmail .= "We'd be glad to hear from you.\n\n";
        $desiredEmail .= "Regards,\n\n";
        $desiredEmail .= $conf['email']['admin_name'];
        $this->assertEqual($email, $desiredEmail);
    }

    /**
     * @todo write test for getNextOperationalInterval()
     */
    function testGetNextOperationalInterval() {
       // missing test
    }

    /**
     * Test Maintenance::getOperationInterval() returns value of
     * $GLOBALS['_MAX']['CONF']['maintenance']['operationInterval']
     */
    function testGetOperationInterval() {
        $this->assertEqual(Maintenance::getOperationInterval(),
            $GLOBALS['_MAX']['CONF']['maintenance']['operationInterval']);
    }

    /**
     * @todo write test for getIntervalsRemaining()
     */
    function testGetIntervalsRemaining() {
        // missing test
    }

}

?>
