#!/usr/bin/php -q
<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
+---------------------------------------------------------------------------+
$Id: summariseDomainPage.php 178 2006-03-14 17:43:49Z andrew@m3.net $
*/

/*
    This script is run daily sometime between midnight and 6am.  It generates statistics
    for Page URL and keyword based reports.  Here is what it does:
    1.  Delete statistics greater than 60 days old:
        a.  Zone inventory by domain and page
        b.  Zone inventory by country
        c.  Zone inventory by source
        d.  Zone inventory by page keyword
    2.  Delete statistics for yesterday (in case it is run more than once):
        a.  Zone inventory by domain and page
        b.  Zone inventory by country
        c.  Zone inventory by source
        d.  Zone inventory by page keyword
    3.  Go through yesterday's log file, and:
        a.  Get zone inventory statistics by domain and page.
        b.  Get zone inventory statistics by country
        c.  Get zone inventory statistics by source
        d.  Get zone inventory statistics by keyword
    4.  Delete forecast statistics:
        a.  Zone inventory by domain and page
        b.  Zone inventory by country
        c.  Zone inventory by source
        d.  Zone inventory by page keyword
    4.  Create tables that forecast by day of week:
        a.  Zone inventory by domain and page
        b.  Zone inventory by country
        c.  Zone inventory by source
        d.  Zone inventory by page keyword
    
    It expects one required and two optional paramaters:
    /usr/bin/php -q /path/to/scripts/maintenance/tools/summariseDomainPage.php {config domain} {action} {params}
    {config domain} is the name of the domain config file to load
    {date} is the date of the stats that you want to re-run the script for in the format "YYYYMMDD"
    
    N.B. : The {date} parameter is the date that the script should have run, not the "yesterday" date
*/
    // Require the initialisation file
    // Done differently from elsewhere so that it works in CLI MacOS X
    $path = dirname(__FILE__);
    define('MAX_PATH', $path . '/../../..');
    
    require_once MAX_PATH . '/init.php';
    require_once MAX_PATH . '/lib/max/other/lib-db.inc.php';
    require_once MAX_PATH . '/www/admin/lib-gui.inc.php';
   
    $iStartTime = time();

    // Set time limit and ignore user abort
    if (!ini_get('safe_mode')) {
        @set_time_limit(600);
        @ignore_user_abort(true);
    }
    if (is_null($GLOBALS['argv'][2])) {

        $date = mktime(0,0,0,date('m'),date('d')-1,date('y'));
        
        // test dates
        //$date = mktime(0,0,0,5,25,2006);
        //$date = mktime(0,0,0,5,26,2006);
        //$date = mktime(0,0,0,5,27,2006);
        //$date = mktime(0,0,0,5,28,2006);
        //$date = mktime(0,0,0,5,29,2006);
        //$date = mktime(0,0,0,5,30,2006);
        //$date = mktime(0,0,0,5,31,2006);
        //$date = mktime(0,0,0,6,1,2006);
        
    } else {
        if (!is_numeric($GLOBALS['argv'][2]) || strlen($GLOBALS['argv'][2]) != 8) {
            quit("If you want to enter a date on the command line, you need to specify it in the format: YYYYMMDD\n\n");
        }
        $year = substr($GLOBALS['argv'][2], 0, 4);
        $month = substr($GLOBALS['argv'][2], 4, 2);
        $day = substr($GLOBALS['argv'][2], 6, 2);
        if (!checkdate($month, $day, $year)) {
            quit("You seem to have entered an invalid date: {$day}/{$month}/{$year}\n\nPlease enter a date in the format YYYYMMDD\n\n");
        }

        $date = mktime(0,0,0,$month,$day-1,$year);
    }
    
    // Define some constants
    $oldDataDays = 60; // delete data older than this amount of days
    $forecastWeeks = 2;  // number of weeks of data to forecast

    // Remove data from the table where it is either:
    // a.  Yesterday's data (we are loading it in)
    // b.  Data from more than 60 days ago (to purge the table)
    
    $aTables = array(
        'data_summary_zone_domain_page',
        'data_summary_zone_country',
        'data_summary_zone_source',
        'data_summary_zone_site_keyword');
        
    foreach ($aTables as $table) {
        _deleteDataSummaryDailyDay($table . '_daily', $date);
        _deleteDataSummaryDailyHistory($table . '_daily', $oldDataDays);
        _deleteDataSummaryForecast($table . '_forecast');
    }
    
    if (date('d', $date) == 1) {
        foreach ($aTables as $table) {
            _deleteDataSummaryMonthlyMonth($table . '_monthly', $date);
        }
    }

    $aZones = _getForecastZones();

    if (!empty($aZones)) {
        _buildDataSummaryZoneDomainPageDaily($aZones, $date);
        _buildDataSummaryZoneCountryDaily($aZones, $date);
        _buildDataSummaryZoneSourceDaily($aZones, $date);
        _buildDataSummaryZoneSiteKeywordDaily($date);
        _buildDataSummaryZoneDomainPageForecast($aZones, $forecastWeeks);
        _buildDataSummaryZoneCountryForecast($aZones, $forecastWeeks);
        _buildDataSummaryZoneSourceForecast($aZones, $forecastWeeks);
        _buildDataSummaryZoneSiteKeywordForecast($aZones, $forecastWeeks);
        if (date('d', $date) == 1) {
            _buildDataSummaryZoneDomainPageMonthly($aZones, $date);
            _buildDataSummaryZoneCountryMonthly($aZones, $date);
            _buildDataSummaryZoneSourceMonthly($aZones, $date);
            _buildDataSummaryZoneSiteKeywordMonthly($aZones, $date);
        }
    }
    
    // Make sure that we clean-up so call quit() with no parameters
    quit();
        
    ////////////////////
    // delete functions
    ////////////////////

    
    function _deleteDataSummaryDailyDay($table, $date)
    {
        global $conf;
        
        $desc = "Deletion of data for the date to be run of the $table table";
    
        $query = "
            DELETE FROM {$conf['table'][$table]}
            WHERE day = '" . date('Y/m/d', $date) . "'
        ";
        $res = phpAds_dbQuery($query);
        if ($res) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _deleteDataSummaryMonthlyMonth($table, $date)
    {
        global $conf;
        
        $desc = "Deletion of data for the date to be run of the $table table";
    
        $query = "
            DELETE FROM {$conf['table'][$table]}
            WHERE yearmonth = DATE_FORMAT(DATE_SUB('" . date('Y-m-d', $date) . "', INTERVAL 1 MONTH), '%Y%m')
        ";
        $res = phpAds_dbQuery($query);
        if ($res) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _deleteDataSummaryDailyHistory($table, $oldDataDays)
    {
        global $conf;
        
        $desc = "Deletion of data older than $oldDataDays days for the date to be run of the $table table";
        
        $query = "
            DELETE FROM {$conf['table'][$table]}
            WHERE day < DATE_SUB(NOW(), INTERVAL " . $oldDataDays . " DAY)
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _deleteDataSummaryForecast($table)
    {
        global $conf;
        
        $desc = "Deletion of forecast data for the $table table";
    
        $query = "
            DELETE FROM {$conf['table'][$table]}
        ";
        $res = phpAds_dbQuery($query);
        if ($res) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }
    
    
    ///////////////////////////
    // summary build functions
    ///////////////////////////
    
    
    function _buildDataSummaryZoneDomainPageDaily($aZones, $date)
    {
        global $conf;

        $desc = 'Summarising data_summary_zone_domain_page_daily';
        
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_domain_page_daily']}
        SELECT
            NULL,
            DATE_FORMAT(date_time, '%Y-%m-%d') AS day,
            zone_id AS zone_id,
            domain AS domain,
            page AS page,
            COUNT(*) AS impressions,
            0 as clicks
        FROM
            {$conf['table']['data_raw_ad_impression']}" . '_' . date('Ymd', $date) . "
        WHERE
            zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day,
            zone_id,
            domain,
            page
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num1 = phpAds_dbAffectedRows();
        } else {
            dbQuit($desc, $query);
        }
        
        // add click totals
                
        $query = "
        SELECT
            DATE_FORMAT(date_time, '%Y-%m-%d') AS day,
            zone_id AS zone_id,
            domain AS domain,
            page AS page,
            COUNT(*) AS clicks
        FROM
            {$conf['table']['data_raw_ad_click']}" . '_' . date('Ymd', $date) . "
        WHERE
            zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day,
            zone_id,
            domain,
            page
        ";
        if ($res1 = phpAds_dbQuery($query)) {
            $aRows = array();
            while ($row = phpAds_dbFetchArray($res1)) {
                $aRows[] = $row;
            }
             
            // add click totals to corresponding rows   
            foreach ($aRows as $rowid => $row) {
                $query = "
                UPDATE {$conf['table']['data_summary_zone_domain_page_daily']}
                SET
                    clicks = ".$row['clicks']."
                WHERE day = '".$row['day']."'
                AND zone_id = ".$row['zone_id']."
                AND domain = '".$row['domain']."'
                AND page = '".$row['page']."'
                ";
                if ($res2 = phpAds_dbQuery($query)) {
                    if (phpAds_dbAffectedRows() > 0) {
                        unset($aRows[$rowid]); // remove this row as it matched an existing table row
                    }
                } else {
                    dbQuit($desc, $query);
                }
            }
            
            // add new rows (i.e. clicks but no impressions)
            if ($rowcnt = count($aRows)) {
                $insertMax = 1000;
                $aQueryInserts = array();
                $queryPre = "INSERT INTO {$conf['table']['data_summary_zone_domain_page_daily']} VALUES ";
                $cnt = 1;
                foreach ($aRows as $rowid => $row) {
                    $aQueryInserts[] = "(NULL, '$row[day]', $row[zone_id], '$row[domain]', '$row[page]', 0, $row[clicks])";
                    if ($cnt == $insertMax) {
                        $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                        if ($res3 = phpAds_dbQuery($query)) {
                           $num2 += phpAds_dbAffectedRows();
                           $cnt = 1; 
                           $aQueryInserts = array();
                        } else {
                            dbQuit($desc, $query);
                        }
                    } else {
                        ++$cnt;
                    }
                }
                // insert any remaining rows
                if (count($aQueryInserts)) {
                    $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                    if ($res3 = phpAds_dbQuery($query)) {
                        $num2 += phpAds_dbAffectedRows();
                    } else {
                        dbQuit($desc, $query);
                    }
                }
            }
            
            $num = $num1 + $num2;
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneCountryDaily($aZones, $date)
    {
        global $conf;
        
        $desc = 'Summarising data_summary_zone_country_daily';
    
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_country_daily']}
        SELECT
            NULL,
            DATE_FORMAT(date_time, '%Y-%m-%d') AS day,
            zone_id AS zone_id,
            country AS country,
            COUNT(*) AS impressions,
            0 as clicks
        FROM
            {$conf['table']['data_raw_ad_impression']}" . '_' . date('Ymd', $date) . "
        WHERE
            zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day,
            zone_id,
            country
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num1 = phpAds_dbAffectedRows();
        } else {
            dbQuit($desc, $query);
        }
        
        // add click totals

        $query = "
        SELECT
            DATE_FORMAT(date_time, '%Y-%m-%d') AS day,
            zone_id AS zone_id,
            country AS country,
            COUNT(*) AS clicks
        FROM
            {$conf['table']['data_raw_ad_click']}" . '_' . date('Ymd', $date) . "
        WHERE
            zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day,
            zone_id,
            country
        ";
        if ($res1 = phpAds_dbQuery($query)) {
            $aRows = array();
            while ($row = phpAds_dbFetchArray($res1)) {
                $aRows[] = $row;
            }

            // add click totals to corresponding rows   
            foreach ($aRows as $rowid => $row) {
                $query = "
                UPDATE {$conf['table']['data_summary_zone_country_daily']}
                SET
                    clicks = ".$row['clicks']."
                WHERE day = '".$row['day']."'
                AND zone_id = ".$row['zone_id']."
                AND country = '".$row['country']."'
                ";
                if ($res2 = phpAds_dbQuery($query)) {
                    if (phpAds_dbAffectedRows() > 0) {
                        unset($aRows[$rowid]); // remove this row as it matched an existing table row
                    }
                } else {
                    dbQuit($desc, $query);
                }
            }
            
            // add new rows (i.e. clicks but no impressions)
            if ($rowcnt = count($aRows)) {
                $insertMax = 1000;
                $aQueryInserts = array();
                $queryPre = "INSERT INTO {$conf['table']['data_summary_zone_country_daily']} VALUES ";
                $cnt = 1;
                foreach ($aRows as $rowid => $row) {
                    $aQueryInserts[] = "(NULL, '$row[day]', $row[zone_id], '$row[country]', 0, $row[clicks])";
                    if ($cnt == $insertMax) {
                        $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                        if ($res3 = phpAds_dbQuery($query)) {
                           $num2 += phpAds_dbAffectedRows();
                           $cnt = 1; 
                           $aQueryInserts = array();
                        } else {
                            dbQuit($desc, $query);
                        }
                    } else {
                        ++$cnt;
                    }
                }
                // insert any remaining rows
                if (count($aQueryInserts)) {
                    $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                    if ($res3 = phpAds_dbQuery($query)) {
                        $num2 += phpAds_dbAffectedRows();
                    } else {
                        dbQuit($desc, $query);
                    }
                }
            }

            $num = $num1 + $num2;
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneSourceDaily($aZones, $date)
    {
        global $conf;
        
        $desc = 'Summarising data_summary_zone_source_daily';
    
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_source_daily']}
        SELECT
            NULL,
            DATE_FORMAT(date_time, '%Y-%m-%d') AS day,
            zone_id AS zone_id,
            channel as source,
            COUNT(*) AS impressions,
            0 as clicks
        FROM
            {$conf['table']['data_raw_ad_impression']}" . '_' . date('Ymd', $date) . "
        WHERE
            zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day,
            zone_id,
            source
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num1 = phpAds_dbAffectedRows();
        } else {
            dbQuit($desc, $query);
        }
        
        // add click totals

        $query = "
        SELECT
            DATE_FORMAT(date_time, '%Y-%m-%d') AS day,
            zone_id AS zone_id,
            channel AS source,
            COUNT(*) AS clicks
        FROM
            {$conf['table']['data_raw_ad_click']}" . '_' . date('Ymd', $date) . "
        WHERE
            zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day,
            zone_id,
            source
        ";
        if ($res1 = phpAds_dbQuery($query)) {
            $aRows = array();
            while ($row = phpAds_dbFetchArray($res1)) {
                $aRows[] = $row;
            }

            // add click totals to corresponding rows   
            foreach ($aRows as $rowid => $row) {
                $query = "
                UPDATE {$conf['table']['data_summary_zone_source_daily']}
                SET
                    clicks = ".$row['clicks']."
                WHERE day = '".$row['day']."'
                AND zone_id = ".$row['zone_id']."
                AND source = '".$row['source']."'
                ";
                if ($res2 = phpAds_dbQuery($query)) {
                    if (phpAds_dbAffectedRows() > 0) {
                        unset($aRows[$rowid]); // remove this row as it matched an existing table row
                    }
                } else {
                    dbQuit($desc, $query);
                }
            }

            // add new rows (i.e. clicks but no impressions)
            if ($rowcnt = count($aRows)) {
                $insertMax = 1000;
                $aQueryInserts = array();
                $queryPre = "INSERT INTO {$conf['table']['data_summary_zone_source_daily']} VALUES ";
                $cnt = 1;
                foreach ($aRows as $rowid => $row) {
                    $aQueryInserts[] = "(NULL, '$row[day]', $row[zone_id], '$row[source]', 0, $row[clicks])";
                    if ($cnt == $insertMax) {
                        $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                        if ($res3 = phpAds_dbQuery($query)) {
                           $num2 += phpAds_dbAffectedRows();
                           $cnt = 1; 
                           $aQueryInserts = array();
                        } else {
                            dbQuit($desc, $query);
                        }
                    } else {
                        ++$cnt;
                    }
                }
                // insert any remaining rows
                if (count($aQueryInserts)) {
                    $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                    if ($res3 = phpAds_dbQuery($query)) {
                        $num2 += phpAds_dbAffectedRows();
                    } else {
                        dbQuit($desc, $query);
                    }
                }
            }            
            
            $num = $num1 + $num2;
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }
    
    
    function _buildDataSummaryZoneSiteKeywordDaily($date)
    {
        global $conf;
        
        $desc = 'Summarising data_summary_zone_site_keyword_daily';

        $query = "
        SELECT
            dszidp.day AS day,
            dszidp.zone_id AS zone_id,
            IF (dszidp.domain LIKE '%cheapflights%',
                'cheapflights',
                IF (dszidp.domain LIKE '%cheapholidaydeals%',
                    'cheapholidaydeals',
                    IF (dszidp.domain LIKE '%cheapshortbreaks%',
                        'cheapshortbreaks',
                        IF (dszidp.domain LIKE '%cheapaccommodation%',
                            'cheapaccommodation',
                            ''
                        )
                    )
                )
            ) AS site,
            IF (dszidp.domain LIKE '%cheapflights%'
                OR dszidp.domain LIKE '%cheapholidaydeals%'
                OR dszidp.domain LIKE '%cheapshortbreaks%'
                OR dszidp.domain LIKE '%cheapaccommodation%',
                LOWER(dszidp.page),
                ''
            ) AS page,
            SUM(impressions) AS impressions,
            SUM(clicks) AS clicks
        FROM
            {$conf['table']['data_summary_zone_domain_page_daily']} AS dszidp,
            {$conf['table']['zones']} AS z,
            {$conf['table']['affiliates']} AS p
        WHERE
            dszidp.zone_id = z.zoneid
            AND z.affiliateid = p.affiliateid
            AND p.agencyid = 21
            AND dszidp.day = '" . date('Y/m/d', $date) . "'
        GROUP BY
            day,
            zone_id,
            site,
            page
        ";
        $res = phpAds_dbQuery($query)
            or dbQuit($desc, $query);

        $aValues = array();
        while ($row = phpAds_dbFetchArray($res)) {
            if (!empty($row['zone_id'])) {
                if (!empty($row['site'])) {
                    if (!empty($row['page'])) {
                        $aKeywords = explode('/', $row['page']);
                        foreach ($aKeywords as $keyword) {
                            // remove items which have % in them (usually encoding errors)
                            // remove items which have . in them (usually the page rather than directory)
                            if (preg_match('#[.%=]+#', $keyword)) {
                                $keyword = '';
                            }
                            // remove non-alphabetic keywords
                            if (!preg_match('#[A-Za-z]+#', $keyword)) {
                                $keyword = '';
                            }
                            // remove short keywords
                            if (strlen($keyword) < 3) {
                                $keyword = '';
                            }
                            if (!empty($keyword)) {
                                $aInventoryForecast[$row['day']][$row['zone_id']][$row['site']][$keyword]['impression_cnt'] += $row['impressions'];
                                $aInventoryForecast[$row['day']][$row['zone_id']][$row['site']][$keyword]['click_cnt'] += $row['clicks'];
                            }
                        }
                    }
                }
            }
        }

        // Perform an insert after this number of rows..
        $insertMax = 1000;
        $num = 0;
        foreach ($aInventoryForecast as $day => $aInventoryForecastDay) {
            foreach ($aInventoryForecastDay as $zoneId => $aInventoryForecastZone) {
                foreach ($aInventoryForecastZone as $site => $aInventoryForecastZoneSite) {
                    $aQueryInserts = array();
                    $queryPre = "INSERT INTO {$conf['table']['data_summary_zone_site_keyword_daily']} VALUES ";
                    $cnt = 1;   
                    foreach ($aInventoryForecastZoneSite as $keyword => $row) {
                        $aQueryInserts[] = "(NULL, '$day', $zoneId,'$site', '$keyword', $row[impression_cnt], $row[click_cnt])";
                        if ($cnt == $insertMax) {
                            $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                            if ($res = phpAds_dbQuery($query)) {
                                $num += phpAds_dbAffectedRows();
                                $cnt = 1; 
                                $aQueryInserts = array();
                            } else {
                                dbQuit($desc, $query);
                            }
                        } else {
                            ++$cnt;
                        }
                    }
                    // insert any remaining rows
                    if (count($aQueryInserts)) {
                        $query = $queryPre.' '.implode(",\n", $aQueryInserts);
                        if ($res = phpAds_dbQuery($query)) {
                            $num += phpAds_dbAffectedRows();
                        } else {
                            dbQuit($desc, $query);
                        }
                    }
                }
            }
        }
        echo $desc." completed. ($num rows)\n";
    }
    
    
    ////////////////////////////
    // forecast build functions
    ////////////////////////////

    
    function _buildDataSummaryZoneDomainPageForecast($aZones, $forecastWeeks)
    {
        global $conf;
        
        $desc = 'Forecasting data_summary_zone_domain_page_forecast';
    
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_domain_page_forecast']}
        SELECT
            NULL,
            DAYOFWEEK(day) AS day_of_week,
            zone_id AS zone_id,
            domain AS domain,
            page AS page,
            SUM(impressions) / $forecastWeeks AS impressions,
            SUM(clicks) / $forecastWeeks AS clicks
        FROM
            {$conf['table']['data_summary_zone_domain_page_daily']}
        WHERE
            day <= DATE_SUB(NOW(), INTERVAL 1 DAY)
            AND day >= DATE_SUB(NOW(), INTERVAL " . ($forecastWeeks * 7) . " DAY)
            AND zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day_of_week,
            zone_id,
            domain,
            page
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneCountryForecast($aZones, $forecastWeeks)
    {
        global $conf;
        
        $desc = 'Forecasting data_summary_zone_country_forecast';

        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_country_forecast']}
        SELECT
            NULL,
            DAYOFWEEK(day) AS day_of_week,
            zone_id AS zone_id,
            country AS country,
            SUM(impressions) / $forecastWeeks AS impressions,
            SUM(clicks) / $forecastWeeks AS clicks
        FROM
            {$conf['table']['data_summary_zone_country_daily']}
        WHERE
            day <= DATE_SUB(NOW(), INTERVAL 1 DAY)
            AND day >= DATE_SUB(NOW(), INTERVAL " . ($forecastWeeks * 7) . " DAY)
            AND zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day_of_week,
            zone_id,
            country
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneSourceForecast($aZones, $forecastWeeks)
    {
        global $conf;
        
        $desc = 'Forecasting data_summary_zone_source_forecast';

        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_source_forecast']}
        SELECT
            NULL,
            DAYOFWEEK(day) AS day_of_week,
            zone_id AS zone_id,
            source AS source,
            SUM(impressions) / $forecastWeeks AS impressions,
            SUM(clicks) / $forecastWeeks AS clicks
        FROM
            {$conf['table']['data_summary_zone_source_daily']}
        WHERE
            day <= DATE_SUB(NOW(), INTERVAL 1 DAY)
            AND day >= DATE_SUB(NOW(), INTERVAL " . ($forecastWeeks * 7) . " DAY)
            AND zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day_of_week,
            zone_id,
            source
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneSiteKeywordForecast($aZones, $forecastWeeks)
    {
        global $conf;
        
        $desc = 'Forecasting data_summary_zone_site_keyword_forecast';

        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_site_keyword_forecast']}
        SELECT
            NULL,
            DAYOFWEEK(day) AS day_of_week,
            zone_id AS zone_id,
            site AS site,
            keyword AS keyword,
            SUM(impressions) / $forecastWeeks AS impressions,
            SUM(clicks) / $forecastWeeks AS clicks
        FROM
            {$conf['table']['data_summary_zone_site_keyword_daily']}
        WHERE
            day <= DATE_SUB(NOW(), INTERVAL 1 DAY)
            AND day >= DATE_SUB(NOW(), INTERVAL " . ($forecastWeeks * 7) . " DAY)
            AND zone_id IN (" . implode(',', $aZones) . ")
        GROUP BY
            day_of_week,
            zone_id,
            site,
            keyword
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    ///////////////////////////
    // monthly build functions
    ///////////////////////////
    
    
    function _buildDataSummaryZoneDomainPageMonthly($aZones, $date)
    {
        global $conf;
        
        $desc = 'Summarising data_summary_zone_domain_page_monthly';
        
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_domain_page_monthly']}
        SELECT
            NULL,
            DATE_FORMAT(day, '%Y%m') AS yearmonth,
            zone_id AS zone_id,
            domain AS domain,
            page AS page,
            SUM(impressions) AS impressions,
            SUM(clicks) AS clicks
        FROM {$conf['table']['data_summary_zone_domain_page_daily']}
        WHERE 
            DATE_FORMAT(day, '%Y%m') = DATE_FORMAT(DATE_SUB('" . date('Y-m-d', $date) . "', INTERVAL 1 MONTH), '%Y%m')
        GROUP BY
            yearmonth,
            zone_id,
            domain,
            page
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneCountryMonthly($aZones, $date)
    {
        global $conf;

        $desc = 'Summarising data_summary_zone_country_monthly';
               
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_country_monthly']}
        SELECT
            NULL,
            DATE_FORMAT(day, '%Y%m') AS yearmonth,
            zone_id AS zone_id,
            country AS country,
            SUM(impressions) AS impressions,
            SUM(clicks) AS clicks
        FROM {$conf['table']['data_summary_zone_country_daily']}
        WHERE 
            DATE_FORMAT(day, '%Y%m') = DATE_FORMAT(DATE_SUB('" . date('Y-m-d', $date) . "', INTERVAL 1 MONTH), '%Y%m')
        GROUP BY
            yearmonth,
            zone_id,
            country
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }

    
    function _buildDataSummaryZoneSourceMonthly($aZones, $date)
    {
        global $conf;
        
        $desc = 'Summarising data_summary_zone_source_monthly';
        
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_source_monthly']}
        SELECT
            NULL,
            DATE_FORMAT(day, '%Y%m') AS yearmonth,
            zone_id AS zone_id,
            source AS source,
            SUM(impressions) AS impressions,
            SUM(clicks) AS clicks
        FROM {$conf['table']['data_summary_zone_source_daily']}
        WHERE 
            DATE_FORMAT(day, '%Y%m') = DATE_FORMAT(DATE_SUB('" . date('Y-m-d', $date) . "', INTERVAL 1 MONTH), '%Y%m')
        GROUP BY
            yearmonth,
            zone_id,
            source
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }


    function _buildDataSummaryZoneSiteKeywordMonthly($aZones, $date)
    {
        global $conf;
        
        $desc = 'Summarising data_summary_zone_site_keyword_monthly';
        
        $query = "
        INSERT INTO {$conf['table']['data_summary_zone_site_keyword_monthly']}
        SELECT
            NULL,
            DATE_FORMAT(day, '%Y%m') AS yearmonth,
            zone_id AS zone_id,
            site AS site,
            keyword AS keyword,
            SUM(impressions) AS impressions,
            SUM(clicks) AS clicks
        FROM {$conf['table']['data_summary_zone_site_keyword_daily']}
        WHERE 
            DATE_FORMAT(day, '%Y%m') = DATE_FORMAT(DATE_SUB('" . date('Y-m-d', $date) . "', INTERVAL 1 MONTH), '%Y%m')
        GROUP BY
            yearmonth,
            zone_id,
            site,
            keyword
        ";
        if ($res = phpAds_dbQuery($query)) {
            $num = phpAds_dbAffectedRows();
            echo $desc." completed. ($num rows)\n";
        } else {
            dbQuit($desc, $query);
        }
    }


    //////////////////
    // misc functions
    //////////////////
    
    
    function _getForecastZones()
    {
        global $conf;
    
        // Get the zones where we are going to look for data (for some reason it is faster to split this out - it is probably an indexing issue)
        // NOTE:  We are currently doing this for any zone where inventory forecasting is required - regardless of zone
        $query = "
        SELECT
            zoneid AS zone_id
        FROM
            {$conf['table']['zones']}
        WHERE
            inventory_forecast_type > 0
        ";
        $res = phpAds_dbQuery($query)
            or dbQuit($desc, $query);
    
        $aZones = array();
        while ($row = phpAds_dbFetchArray($res)) {
            $aZones[] = $row['zone_id'];
        }
        
        return $aZones;
    }

    
    function dbQuit($desc, $query) {
        $message = $desc." failed:\n";
        $message .= "\$query = '{$query}'\n\n";
        $message .= "mysql_error() = '" . mysql_error() . "\n\n";
        quit($message);
    }
    
    
    function quit($message = false) {
        global $conf, $fh, $iStartTime;
        // This is a safe-exit function that will take an optional message which will be output to STDOUT
        // It also cleans up after itself so that things like the LOCK file are removed.
        if ($message) {
            echo $message;
        }

        if (is_resource($fh)) {
            fclose($fh);
            unlink($conf['table']['lockfile']);
        }

        $iEndTime = time();
        $iDuration = $iEndTime - $iStartTime;
        echo "\nScript duration = {$iDuration}s\n";
        exit;
    }

?>
